/*
 * JBoss, Home of Professional Open Source.
 * Copyright 2008, Red Hat Middleware LLC, and individual contributors
 * as indicated by the @author tags. See the copyright.txt file in the
 * distribution for a full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jboss.aspects.currentinvocation.test.basic;

import org.jboss.aop.joinpoint.Invocation;
import org.jboss.aspects.currentinvocation.CurrentInvocation;

/**
 * PojoBean
 * 
 * A Simple POJO to act as target for Current Invocation
 * Interception for use in testing
 * 
 * @author <a href="mailto:andrew.rubinger@jboss.org">ALR</a>
 * @version $Revision: $
 */
public class PojoBean
{
   /**
    * Returns whether or not the current invocation can be 
    * obtained from the registry, validating that the Interceptor
    * worked as expected
    * @return
    */
   public boolean canGetCurrentInvocation()
   {
      Invocation invocation = CurrentInvocation.getCurrentInvocation();
      return invocation != null;
   }

   /**
    * Throw an exception
    */
   public void throwException() throws TestException
   {
      // Ensure current invocation is available
      this.ensureCurrentInvocationIsAvailable();

      // Throw the expected TestException
      throw new TestException("ALR : Carlo :: Einstein : Paris Hilton");
   }

   /**
    * Define a bind for interception only, and
    * wait until notified by another Thread
    * 
    * @param millis
    */
   public void suspend()
   {
      // Ensure current invocation is available
      this.ensureCurrentInvocationIsAvailable();

      try
      {
         wait();
      }
      catch (InterruptedException e)
      {
         throw new RuntimeException(e);
      }

   }

   /**
    * Throw a RuntimeException if no Current Invocation
    * is in scope
    */
   private void ensureCurrentInvocationIsAvailable()
   {
      if (CurrentInvocation.getCurrentInvocation() == null)
      {
         throw new RuntimeException("Current Invocation not found");
      }
   }
}
